@@ -6,26 +6,26 @@
 #ifndef _IP_VS_H
 #define _IP_VS_H
 
-#define IP_VS_VERSION_CODE            0x000005
+#define IP_VS_VERSION_CODE            0x000100
 #define NVERSION(version)                       \
         (version >> 16) & 0xFF,                 \
         (version >> 8) & 0xFF,                  \
         version & 0xFF                       
 
 /*
- *     Virtual Service Flags
+ *      Virtual Service Flags
  */
 #define IP_VS_SVC_F_PERSISTENT        0x0001    /* persistent port */
 #define IP_VS_SVC_F_HASHED            0x0002    /* hashed entry */
 
 /*
- *     Destination Server Flags
+ *      Destination Server Flags
  */
 #define IP_VS_DEST_F_AVAILABLE        0x0001    /* Available tag */
 
 /*
  * The default IP_VS_TEMPLATE_TIMEOUT is a little larger than average
- * connection time plus MASQUERADE_EXPIRE_TCP_FIN(2*60*HZ). Because the
+ * connection time plus IPVS TCP FIN timeout(2*60*HZ). Because the
  * template won't be released until its controlled masq entries are
  * expired.
  * If IP_VS_TEMPLATE_TIMEOUT is too less, the template will soon expire
@@ -61,18 +61,12 @@ struct ip_vs_rule_user {
 };
 
 
-#define MASQUERADE_EXPIRE_TCP     15*60*HZ
-#define MASQUERADE_EXPIRE_TCP_FIN  2*60*HZ
-#define MASQUERADE_EXPIRE_UDP      5*60*HZ
-/* 
- * ICMP can no longer be modified on the fly using an ioctl - this
- * define is the only way to change the timeouts 
+/*
+ *      IPVS socket options
  */
-#define MASQUERADE_EXPIRE_ICMP      125*HZ
+#define IP_VS_BASE_CTL  	(64+1024+64)            /* base */
 
-#define IP_VS_BASE_CTL  	(64+1024+64)    /* base for IPVS socket options */
-
-#define IP_VS_SO_SET_NONE	IP_VS_BASE_CTL	         /* just peek */
+#define IP_VS_SO_SET_NONE	IP_VS_BASE_CTL	        /* just peek */
 #define IP_VS_SO_SET_INSERT	(IP_VS_BASE_CTL+1)
 #define IP_VS_SO_SET_ADD	(IP_VS_BASE_CTL+2)
 #define IP_VS_SO_SET_EDIT	(IP_VS_BASE_CTL+3)
@@ -105,10 +99,11 @@ struct ip_vs_rule_user {
 #ifdef __KERNEL__
 
 #include <linux/config.h>
-#include <linux/list.h>                 /* for list_head */
-#include <linux/spinlock.h>             /* for rwlock_t */
-#include <linux/skbuff.h>               /* for sk_buff */
-#include <asm/atomic.h>                 /* for atomic_t */
+#include <linux/list.h>                 /* for struct list_head */
+#include <linux/spinlock.h>             /* for struct rwlock_t */
+#include <linux/skbuff.h>               /* for struct sk_buff */
+#include <linux/ip.h>                   /* for struct iphdr */
+#include <asm/atomic.h>                 /* for struct atomic_t */
 
 
 /*  #define CONFIG_IP_VS_DEBUG */
@@ -154,9 +149,12 @@ extern int ip_vs_get_debug_level(void);
 #define FTPDATA  __constant_htons(20)
 
 
-/* /proc/sys/net/ipv4/vs */
-enum
-{
+/*
+ *  IPVS sysctl variables under the /proc/sys/net/ipv4/vs/
+ */
+#define NET_IPV4_VS              21
+
+enum {
 	NET_IPV4_VS_DEBUG_LEVEL=1,
 	NET_IPV4_VS_AMEMTHRESH=2,
 	NET_IPV4_VS_AMDROPRATE=3,
@@ -175,6 +173,8 @@ enum
 	NET_IPV4_VS_TO_SA=16,
 	NET_IPV4_VS_TO_UDP=17,
 	NET_IPV4_VS_TO_ICMP=18,
+	NET_IPV4_VS_LBLC_EXPIRE=19,
+	NET_IPV4_VS_LBLCR_EXPIRE=19,
 };
 
 enum {
@@ -319,7 +319,8 @@ struct ip_vs_scheduler {
         int (*update_service)(struct ip_vs_service *svc);
 
 	/* selecting a server from the given service */
-	struct ip_vs_dest* (*schedule)(struct ip_vs_service *svc);
+	struct ip_vs_dest* (*schedule)(struct ip_vs_service *svc,
+                                       struct iphdr *iph);
 };
 
 
