@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_conn.c,v 1.5 2000/09/06 08:36:36 wensong Exp $
+ * Version:     $Id: ip_vs_conn.c,v 1.6 2000/11/26 04:35:39 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -917,8 +917,10 @@ static inline int todrop_entry(struct ip_vs_conn *cp)
 	if (cp->timeout+jiffies-cp->timer.expires < 60*HZ)
 		return 0;
 
+        /* Don't drop the entry if its number of incoming packets is not
+           located in [0, 8] */
 	i = atomic_read(&cp->in_pkts);
-	if (i > 8) return 0;
+	if (i > 8 || i < 0) return 0;
 
 	if (!todrop_rate[i]) return 0;
 	if (--todrop_counter[i] > 0) return 0;
