@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_core.c,v 1.7 2000/09/06 08:36:36 wensong Exp $
+ * Version:     $Id: ip_vs_core.c,v 1.8 2000/11/26 04:35:39 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -210,7 +210,7 @@ ip_vs_sched_persist(struct ip_vs_service *svc, struct iphdr *iph)
 			 * No template found or the dest of the connection
 			 * template is not available.
 			 */
-			dest = svc->scheduler->schedule(svc);
+			dest = svc->scheduler->schedule(svc, iph);
 			if (dest == NULL) {
 				IP_VS_DBG(1, "P-schedule: no dest found.\n");
 				return NULL;
@@ -278,7 +278,7 @@ ip_vs_sched_persist(struct ip_vs_service *svc, struct iphdr *iph)
 			if (svc->port)
 				return NULL;
 
-			dest = svc->scheduler->schedule(svc);
+			dest = svc->scheduler->schedule(svc, iph);
 			if (dest == NULL) {
 				IP_VS_DBG(1, "P-schedule: no dest found.\n");
 				return NULL;
@@ -385,7 +385,7 @@ struct ip_vs_conn *ip_vs_schedule(struct ip_vs_service *svc, struct iphdr *iph)
 		return NULL;
 	}
 
-	dest = svc->scheduler->schedule(svc);
+	dest = svc->scheduler->schedule(svc, iph);
 	if (dest == NULL) {
 		IP_VS_DBG(1, "Schedule: no dest found.\n");
 		return NULL;
@@ -1396,17 +1396,18 @@ static unsigned int ip_vs_in(unsigned int hooknum,
 }
 
 
-/* Before packet filtering, forward packet through VS/DR, VS/TUN,
-   or VS/NAT(change destination) */
+/* After packet filtering, forward packet through VS/DR, VS/TUN,
+   or VS/NAT(change destination), so that filtering rules can be
+   applied to IPVS. */
 static struct nf_hook_ops ip_vs_in_ops = {
         { NULL, NULL },
-        ip_vs_in, PF_INET, /*  NF_IP_PRE_ROUTING */NF_IP_LOCAL_IN, -100
+        ip_vs_in, PF_INET, /* NF_IP_PRE_ROUTING */NF_IP_LOCAL_IN, 100
 };
 
 /* After packet filtering, change source only for VS/NAT */
 static struct nf_hook_ops ip_vs_out_ops = {
         { NULL, NULL },
-        ip_vs_out, PF_INET, /*  NF_IP_POST_ROUTING */NF_IP_FORWARD, 100
+        ip_vs_out, PF_INET, /* NF_IP_POST_ROUTING */NF_IP_FORWARD, 100
 };
 
 
