@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.5 2000/09/06 08:36:36 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.6 2000/11/26 04:35:39 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -868,7 +868,7 @@ static int ip_vs_del_dest(struct ip_vs_service *svc,struct ip_vs_rule_user *ur)
 	/*
 	 *	Unlink dest from the service
 	 */
-	__ip_vs_unlink_dest(svc,dest,1);
+	__ip_vs_unlink_dest(svc, dest, 1);
 	write_unlock_bh(&__ip_vs_svc_lock);
 
 	/*
@@ -976,56 +976,69 @@ static int ip_vs_add_service(struct ip_vs_rule_user *ur)
 static int ip_vs_edit_service(struct ip_vs_service *svc,
 			      struct ip_vs_rule_user *ur)
 {
-	struct ip_vs_scheduler *sched, *old_sched;
+        struct ip_vs_scheduler *sched, *old_sched;
+        int ret = 0;
 
-	/*
-	 * Lookup the scheduler, by 'ur->sched_name'
-	 */
-	sched = ip_vs_lookup_scheduler(ur->sched_name);
-	if (sched == NULL) {
-		IP_VS_INFO("Scheduler module ip_vs_%s.o not found\n",
-			   ur->sched_name);
-		return -ENOENT;
-	}
+        /*
+         * Lookup the scheduler, by 'ur->sched_name'
+         */
+        sched = ip_vs_lookup_scheduler(ur->sched_name);
+        if (sched == NULL) {
+                IP_VS_INFO("Scheduler module ip_vs_%s.o not found\n",
+                           ur->sched_name);
+                return -ENOENT;
+        }
 
-	write_lock_bh(&__ip_vs_svc_lock);
+        write_lock_bh(&__ip_vs_svc_lock);
 
-	/*
-	 * Wait until all svc users go away
-	 */
-	while (atomic_read(&svc->refcnt) > 1) {};
+        /*
+         * Wait until all svc users go away
+         */
+        while (atomic_read(&svc->refcnt) > 1) {};
 
-	/*
-	 * Set the flags and timeout value
-	 */
-	svc->flags = ur->vs_flags | IP_VS_SVC_F_HASHED;
-	svc->timeout = ur->timeout;
-	svc->netmask = ur->netmask;
+        /*
+         * Set the flags and timeout value
+         */
+        svc->flags = ur->vs_flags | IP_VS_SVC_F_HASHED;
+        svc->timeout = ur->timeout;
+        svc->netmask = ur->netmask;
 
-	/*
-	 * Unbind the old scheduler
-	 */
-	old_sched = svc->scheduler;
-	ip_vs_unbind_scheduler(svc);
+        old_sched = svc->scheduler;
+        if (sched != old_sched) {
+                /*
+                 * Unbind the old scheduler
+                 */
+                if ((ret = ip_vs_unbind_scheduler(svc))) {
+                        old_sched = sched;
+                        goto out;
+                }
+                
+                /*
+                 * Bind the new scheduler
+                 */
+                if ((ret = ip_vs_bind_scheduler(svc, sched))) {
+                        /*
+                         * If ip_vs_bind_scheduler fails, restore the old
+                         * scheduler.
+                         * The main reason of failure is out of memory.
+                         *
+                         * The question is if the old scheduler can be
+                         * restored all the time. TODO: if it cannot be
+                         * restored some time, we must delete the service, 
+                         * otherwise the system may crash.
+                         */
+                        ip_vs_bind_scheduler(svc, old_sched);
+                        old_sched = sched;
+                }
+        }
 
-	/*
-	 * Bind the new scheduler
-	 */
-	ip_vs_bind_scheduler(svc, sched);
-	/*
-	** If ip_vs_bind_scheduler fails (which is
-	** possible) we just crash!!! TODO: restore
-	** the old scheduler or delete this service,
-	** to release some memory :) The preferred
-	** result is to return -ENOMEM to the user
-	** without touching the svc!
-	*/
+  out:
+        write_unlock_bh(&__ip_vs_svc_lock);
 
-	write_unlock_bh(&__ip_vs_svc_lock);
+        if (old_sched)
+                __MOD_DEC_USE_COUNT(old_sched->module);
 
-	if (old_sched) __MOD_DEC_USE_COUNT(old_sched->module);
-
-	return 0;
+        return ret;
 }
 
 
@@ -1035,27 +1048,27 @@ static int ip_vs_edit_service(struct ip_vs_service *svc,
  */
 static int __ip_vs_del_service(struct ip_vs_service *svc)
 {
-	struct list_head *l;
-	struct ip_vs_dest *dest;
-	struct ip_vs_scheduler *old_sched;
+        struct list_head *l;
+        struct ip_vs_dest *dest;
+        struct ip_vs_scheduler *old_sched;
 
-	/*
-	 * Unbind scheduler
-	 */
-	old_sched = svc->scheduler;
-	ip_vs_unbind_scheduler(svc);
-	if (old_sched)
+        /*
+         * Unbind scheduler
+         */
+        old_sched = svc->scheduler;
+        ip_vs_unbind_scheduler(svc);
+        if (old_sched)
                 __MOD_DEC_USE_COUNT(old_sched->module);
 
-	/*
-	 *    Unlink the whole destination list
-	 */
-	l = &svc->destinations;
-	while (l->next != l) {
-		dest = list_entry(l->next, struct ip_vs_dest, n_list);
-		__ip_vs_unlink_dest(svc, dest, 0);
-		__ip_vs_del_dest(dest);
-	}
+        /*
+         *    Unlink the whole destination list
+         */
+        l = &svc->destinations;
+        while (l->next != l) {
+                dest = list_entry(l->next, struct ip_vs_dest, n_list);
+                __ip_vs_unlink_dest(svc, dest, 0);
+                __ip_vs_del_dest(dest);
+        }
 
         /*
          *    Update the virtual service counters
@@ -1066,11 +1079,11 @@ static int __ip_vs_del_service(struct ip_vs_service *svc)
                 atomic_dec(&ip_vs_nullsvc_counter);
 
         /*
-	 * Free the service
-	 */
-	kfree(svc);
-	MOD_DEC_USE_COUNT;
-	return 0;
+         *    Free the service
+         */
+        kfree(svc);
+        MOD_DEC_USE_COUNT;
+        return 0;
 }
 
 /*
@@ -1078,23 +1091,23 @@ static int __ip_vs_del_service(struct ip_vs_service *svc)
  */
 static int ip_vs_del_service(struct ip_vs_service *svc)
 {
-	if (svc == NULL)
-		return -EEXIST;
+        if (svc == NULL)
+                return -EEXIST;
 
-	/*
-	 * Unhash it from the service table
-	 */
-	write_lock_bh(&__ip_vs_svc_lock);
-	ip_vs_svc_unhash(svc);
+        /*
+         * Unhash it from the service table
+         */
+        write_lock_bh(&__ip_vs_svc_lock);
+        ip_vs_svc_unhash(svc);
 
-	/*
-	 * Wait until all svc users go away
-	 */
-	while (atomic_read(&svc->refcnt) > 1) {};
+        /*
+         * Wait until all svc users go away
+         */
+        while (atomic_read(&svc->refcnt) > 1) {};
 
-	write_unlock_bh(&__ip_vs_svc_lock);
+        write_unlock_bh(&__ip_vs_svc_lock);
 
-	return __ip_vs_del_service(svc);
+        return __ip_vs_del_service(svc);
 }
 
 
@@ -1103,71 +1116,72 @@ static int ip_vs_del_service(struct ip_vs_service *svc)
  */
 static int ip_vs_flush(void)
 {
-	int idx;
-	struct ip_vs_service *svc;
-	struct list_head *l;
+        int idx;
+        struct ip_vs_service *svc;
+        struct list_head *l;
 
-	/*
-	 * Flush the service table hashed by <protocol,addr,port>
-	 */
-	for(idx = 0; idx < IP_VS_SVC_TAB_SIZE; idx++) {
-		l = &ip_vs_svc_table[idx];
-		while (l->next != l) {
-			svc = list_entry(l->next,struct ip_vs_service,s_list);
-			write_lock_bh(&__ip_vs_svc_lock);
-			ip_vs_svc_unhash(svc);
-			/*
-			 *	Wait until all svc users go away
-			 */
-			while (atomic_read(&svc->refcnt) > 1) {};
-			__ip_vs_del_service(svc);
-			write_unlock_bh(&__ip_vs_svc_lock);
-		}
-	}
+        /*
+         * Flush the service table hashed by <protocol,addr,port>
+         */
+        for(idx = 0; idx < IP_VS_SVC_TAB_SIZE; idx++) {
+                l = &ip_vs_svc_table[idx];
+                while (l->next != l) {
+                        svc = list_entry(l->next,struct ip_vs_service,s_list);
+                        write_lock_bh(&__ip_vs_svc_lock);
+                        ip_vs_svc_unhash(svc);
+                        /*
+                         *	Wait until all svc users go away
+                         */
+                        while (atomic_read(&svc->refcnt) > 1) {};
+                        __ip_vs_del_service(svc);
+                        write_unlock_bh(&__ip_vs_svc_lock);
+                }
+        }
 
-	/*
-	 * Flush the service table hashed by fwmark
-	 */
-	for(idx = 0; idx < IP_VS_SVC_TAB_SIZE; idx++) {
-		l = &ip_vs_svc_fwm_table[idx];
-		while (l->next != l) {
-			svc = list_entry(l->next,struct ip_vs_service,f_list);
-			write_lock_bh(&__ip_vs_svc_lock);
-			ip_vs_svc_unhash(svc);
-			/*
-			 * Wait until all svc users go away
-			 */
-			while (atomic_read(&svc->refcnt) > 1) {};
-			__ip_vs_del_service(svc);
-			write_unlock_bh(&__ip_vs_svc_lock);
-		}
-	}
+        /*
+         * Flush the service table hashed by fwmark
+         */
+        for(idx = 0; idx < IP_VS_SVC_TAB_SIZE; idx++) {
+                l = &ip_vs_svc_fwm_table[idx];
+                while (l->next != l) {
+                        svc = list_entry(l->next,struct ip_vs_service,f_list);
+                        write_lock_bh(&__ip_vs_svc_lock);
+                        ip_vs_svc_unhash(svc);
+                        /*
+                         * Wait until all svc users go away
+                         */
+                        while (atomic_read(&svc->refcnt) > 1) {};
+                        __ip_vs_del_service(svc);
+                        write_unlock_bh(&__ip_vs_svc_lock);
+                }
+        }
 
-	return 0;
+        return 0;
 }
 
 
 static int ip_vs_sysctl_defense_mode(ctl_table *ctl, int write,
 	struct file * filp, void *buffer, size_t *lenp)
 {
-	int *valp = ctl->data;
-	int val = *valp;
-	int ret;
+        int *valp = ctl->data;
+        int val = *valp;
+        int ret;
 
-	ret = proc_dointvec(ctl, write, filp, buffer, lenp);
-	if (write && (*valp != val)) {
-		if ((*valp < 0) || (*valp > 3)) {
-			/* Restore the correct value */
-			*valp = val;
-		} else {
-			local_bh_disable();
-			update_defense_level();
-			local_bh_enable();
-		}
-	}
-	return ret;
+        ret = proc_dointvec(ctl, write, filp, buffer, lenp);
+        if (write && (*valp != val)) {
+                if ((*valp < 0) || (*valp > 3)) {
+                        /* Restore the correct value */
+                        *valp = val;
+                } else {
+                        local_bh_disable();
+                        update_defense_level();
+                        local_bh_enable();
+                }
+        }
+        return ret;
 }
 
+
 /*
  *      IPVS sysctl table
  */
@@ -1179,7 +1193,6 @@ struct ip_vs_sysctl_table {
 	ctl_table root_dir[2];
 };
 
-#define NET_IPV4_VS              21
 
 static struct ip_vs_sysctl_table ipv4_vs_table = {
         NULL,
