@@ -1,7 +1,7 @@
 /*
  *      ipvsadm - IP Virtual Server ADMinistration program
  *
- *      Version: $Id: ipvsadm.c,v 1.3 2000/09/06 08:36:36 wensong Exp $
+ *      Version: $Id: ipvsadm.c,v 1.4 2000/11/26 04:35:39 wensong Exp $
  *
  *      Authors: Wensong Zhang <wensong@iinchina.net>
  *               Peter Kese <peter.kese@ijs.si>
@@ -1131,7 +1131,7 @@ void usage_exit(const char *program, const int exit_status) {
                 "  --tcp-service  -t service-address   service-address is host[:port]\n"
                 "  --udp-service  -u service-address   service-address is host[:port]\n"
                 "  --fwmark-service  -f fwmark         fwmark is an integer greater than zero\n"
-                "  --scheduler    -s <scheduler>       one of rr|wrr|lc|wlc,\n"
+                "  --scheduler    -s <scheduler>       one of rr|wrr|lc|wlc|lblc,\n"
                 "                                      the default scheduler is %s.\n"
                 "  --persistent   -p [timeout]         persistent service\n"
                 "  --netmask      -M [netmask]         persistent granularity mask\n"
